package dev.fyloz.colorrecipesexplorer.utils

import javax.servlet.http.HttpServletResponse

private const val defaultCookieMaxAge = 3600L
private const val defaultCookieHttpOnly = true
private const val defaultCookieSameSite = true
private const val defaultCookieSecure = true

data class CookieOptions(
    /** HTTP Only cookies cannot be access by Javascript clients. */
    var httpOnly: Boolean = defaultCookieHttpOnly,

    /** SameSite cookies are only sent in requests to their origin location. */
    var sameSite: Boolean = defaultCookieSameSite,

    /** Secure cookies are only sent in HTTPS requests. */
    var secure: Boolean = defaultCookieSecure,

    /** Cookie's maximum age in seconds. */
    var maxAge: Long = defaultCookieMaxAge
)

private enum class CookieOption(val optionName: String) {
    HTTP_ONLY("HttpOnly"),
    SAME_SITE("SameSite"),
    SECURE("Secure"),
    MAX_AGE("Max-Age")
}

fun HttpServletResponse.addCookie(name: String, value: String, optionsBuilder: CookieOptions.() -> Unit) {
    this.addHeader("Set-Cookie", buildCookie(name, value, optionsBuilder))
}

private fun buildCookie(name: String, value: String, optionsBuilder: CookieOptions.() -> Unit): String {
    val options = CookieOptions().apply(optionsBuilder)
    val cookie = StringBuilder("$name=$value;")

    fun addBoolOption(option: CookieOption, enabled: Boolean) {
        if (enabled) {
            cookie.append("${option.optionName};")
        }
    }

    fun addOption(option: CookieOption, value: Any) {
        cookie.append("${option.optionName}=$value;")
    }

    addBoolOption(CookieOption.HTTP_ONLY, options.httpOnly)
    addBoolOption(CookieOption.SAME_SITE, options.sameSite)
    addBoolOption(CookieOption.SECURE, options.secure)
    addOption(CookieOption.MAX_AGE, options.maxAge)

    return cookie.toString()
}
